﻿using HarmonyLib;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;

public class FireworkLauncher : Mod
{
    private bool isInGame;
    private SO_ColorValue[] colorValues;
    public static Item_Base launcherItem;
    Harmony harmony;
    public void Awake()
    {
        isInGame = false;
        StartCoroutine("Init");
    }

    void CreateFireworkLauncher()
    {
        Item_Base baseItem = ItemManager.GetItemByName("Placeable_Firework");
        launcherItem = baseItem.Clone(6969, "Placeable_Firework_Launcher");
        launcherItem.name = launcherItem.UniqueName;
        launcherItem.settings_Inventory.LocalizationTerm = "";
        launcherItem.settings_Inventory.DisplayName = "Firework Launcher";
        launcherItem.settings_Inventory.Description = "Launches fireworks remotely.";
        launcherItem.settings_recipe = new ItemInstance_Recipe(CraftingCategory.Other, false, false, "Firework Launcher", 0);

        // Set recipe - not working yet, doesn't appear in build menu
        launcherItem.settings_recipe.SetRecipe(new CostMultiple[]
        {
            new CostMultiple(new Item_Base[] {ItemManager.GetItemByName("Scrap")}, 6),
            new CostMultiple(new Item_Base[] {ItemManager.GetItemByName("TitaniumIngot")}, 1),
        }, 2);

        launcherItem.settings_recipe.LearnedViaBlueprint = true;

        var prefabsBase = Traverse.Create(launcherItem.settings_buildable).Field<Block[]>("blockPrefabs");
        var newPrefabs = new Block[prefabsBase.Value.Length];

        // Thank you again Aidanamite
        var prefabParent = new GameObject("prefabParent").transform;
        prefabParent.gameObject.SetActive(false);
        DontDestroyOnLoad(prefabParent.gameObject);

        for (int i = 0; i < prefabsBase.Value.Length; i++)
        {
            newPrefabs[i] = Instantiate(prefabsBase.Value[i], new Vector3(float.NaN, float.NaN, float.NaN), new Quaternion(0, 0, 0, 1), prefabParent);

            newPrefabs[i].gameObject.AddComponent<Block_Firework_Launcher>();

            newPrefabs[i].itemToReturnOnDestroy = launcherItem;
            newPrefabs[i].buildableItem = launcherItem;

            newPrefabs[i].name = launcherItem.UniqueName;
        }

        prefabsBase.Value = newPrefabs;
        Traverse.Create(launcherItem.settings_Inventory).Field("stackSize").SetValue(10);

        foreach (var type in Resources.FindObjectsOfTypeAll<SO_BlockQuadType>())
            if (type.AcceptsBlock(baseItem))
                Traverse.Create(type).Field("acceptableBlockTypes").GetValue<List<Item_Base>>().Add(launcherItem);

        RAPI.RegisterItem(launcherItem);
        harmony = new Harmony("com.boebie.FireworkLauncher");
        harmony.PatchAll();

        Debug.Log("FireworkLauncher has been loaded!");
    }
    public override void WorldEvent_WorldLoaded()
    {
        isInGame = true;
    }

    public override void WorldEvent_WorldUnloaded()
    {
        isInGame = false;
    }

    public IEnumerator Init()
    {
        CreateFireworkLauncher();

        while (RAPI.GetLocalPlayer() == null)
            yield return new WaitForEndOfFrame();

        isInGame = true;
        colorValues = Resources.LoadAll<SO_ColorValue>("Colors");
    }

    public void OnModUnload()
    {
        isInGame = false;
        if (launcherItem)
        {
            harmony.UnpatchAll(harmony.Id);
            Debug.Log("Mod FireworkLauncher has been unloaded!");
        }
    }

    void Update()
    {
        if (isInGame && Input.GetKeyDown(KeyCode.F11))
        {
            Network_Player player = RAPI.GetLocalPlayer();
            if (player.FireworkHand != null)
            {
                foreach (var g in FindObjectsOfType<Block_Firework_Launcher>())
                {
                    int fireworkColor = UnityEngine.Random.Range(0, colorValues.Length);
                    GameObject fireworkModel = Traverse.Create(g.GetComponent<Block_Firework>()).Field("fireworkModel").GetValue() as GameObject;
                    player.FireworkHand.LaunchFireworkNetworked(fireworkModel.transform.position, fireworkModel.transform.up, false, fireworkColor);
                }
            }
        }
    }
}

public class Block_Firework_Launcher : Block
{
    public static List<Block_Firework_Launcher> launchers = new List<Block_Firework_Launcher>();
    public void OnBlockPlaced()
    {
        launchers.Add(this);
        Traverse.Create(this.GetComponent<Block_Firework>()).Field("hasBeenUsed").SetValue(true);
    }

    public void OnDestroy()
    {
        if (launchers.Contains(this))
            launchers.Remove(this);
    }
}

// Hopefully Aidanamite doesn't mind me using this code :)
static class ExtentionMethods
{
    public static Item_Base Clone(this Item_Base source, int uniqueIndex, string uniqueName)
    {
        Item_Base item = ScriptableObject.CreateInstance<Item_Base>();
        item.Initialize(uniqueIndex, uniqueName, source.MaxUses);
        item.settings_buildable = source.settings_buildable.Clone();
        item.settings_consumeable = source.settings_consumeable.Clone();
        item.settings_cookable = source.settings_cookable.Clone();
        item.settings_equipment = source.settings_equipment.Clone();
        item.settings_Inventory = source.settings_Inventory.Clone();
        item.settings_recipe = source.settings_recipe.Clone();
        item.settings_usable = source.settings_usable.Clone();
        return item;
    }

    public static void SetRecipe(this ItemInstance_Recipe item, CostMultiple[] cost, int amountToCraft = 1)
    {
        Traverse.Create(item).Field("amountToCraft").SetValue(amountToCraft);
        item.NewCost = cost;
    }
}